#import "GameSocket.h"

// CocoaLumberjack
#import "DDLog.h"
#import "DDTTYLogger.h"

#import "GameSocketSocket.h"
#import "GameSocketSerializer.h"
#import "NSString+UUID.h"

static int ddLogLevel = LOG_LEVEL_VERBOSE;

// Declaring internal Properties
@interface GameSocket ()
  @property (nonatomic, strong) GameSocketSocket *socket;
@end

@implementation GameSocket

// Public Properties
@synthesize uuid, delegate;

// Internal Properties
@synthesize socket;

- (BOOL) start {
  [self.socket start];
  return YES;
}

- (BOOL) stop {
  [self.socket stop];
  return YES;
}

- (void) sendEvent:(GameSocketEvent*)event {
  GameSocketDatagram *datagram = [self eventToDatagram:event];
  [self.socket sendDatagram:datagram];
}

/*** Internal Methods ***/

- (GameSocketDatagram*) eventToDatagram:(GameSocketEvent*)event {
  NSArray *keys = [NSArray arrayWithObjects:@"sender_id", @"kind", @"data", nil];
  NSArray *values = [NSArray arrayWithObjects:self.uuid, event.kind, event.data, nil];
  NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:values forKeys:keys];
  DDLogVerbose(@"Preparing Datagram for %@", dictionary);
  GameSocketDatagram* datagram = [GameSocketDatagram new];
  datagram.endpoint = self.serverEndpoint;
  datagram.port = self.serverPort;
  datagram.payload = [GameSocketSerializer pack:dictionary];
  return datagram;
}

/*** Internal Getters ***/

- (GameSocketSocket*) socket {
  if (socket) return socket;
  DDLogVerbose(@"Initializing GameSocketSocket...");
  socket = [GameSocketSocket new];
  socket.delegate = self;
  return socket;
}

- (NSString*) uuid {
  if (uuid) return uuid;
  uuid = [NSString stringWithNewUUID];
  DDLogVerbose(@"Initialized random UUID %@", uuid);
  return uuid;
}

/*** Public Setters/Getters ***/

- (void) setServerEndpoint:(NSString*) newServerEndpoint {
  self.socket.serverEndpoint = newServerEndpoint;
}

- (NSString*) serverEndpoint {
  return self.socket.serverEndpoint;
}

- (void) setServerPort:(uint16_t)newServerPort {
  self.socket.serverPort = newServerPort;
}

- (uint16_t) serverPort {
  return self.socket.serverPort;
}

- (void) setPort:(uint16_t)newPort {
  self.socket.port = newPort;
}

- (uint16_t) port {
  return self.socket.port;
}

/*** Delegation Methods ***/

- (void) didReceiveDatagram:(GameSocketDatagram*)datagram {
  DDLogVerbose(@"Received Datagram...");
  if ([self.delegate respondsToSelector:@selector(didReceiveEvent:)]) {
    NSDictionary *dictionary = [GameSocketSerializer unpack:datagram.payload];
    GameSocketEvent *event = [GameSocketEvent eventOfKind:[dictionary valueForKey:@"kind"] andData:[dictionary valueForKey:@"data"]];
    [self.delegate didReceiveEvent:event];
  }
}

@end
